"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const inversify_1 = require("inversify");
const flow_1 = require("../interfaces/flow");
const git_1 = require("../../utils/git");
const cacheResolver_1 = require("../../resolvers/cacheResolver");
const ciResolver_1 = require("../../resolvers/ciResolver");
const testResolver_1 = require("../../resolvers/testResolver");
let Flow = class Flow extends flow_1.IFlow {
    constructor(gitCheckout, cacheResolver, ciResolver, testResolver) {
        super();
        this.gitCheckout = gitCheckout;
        this.cacheResolver = cacheResolver;
        this.ciResolver = ciResolver;
        this.testResolver = testResolver;
    }
    async runFlow({ repo, targetNode, pkg, workDir }) {
        const cacheResult = await this.cacheResolver.resolve({
            targetNode,
            repo: {
                name: pkg.name,
                version: pkg.version,
            },
        });
        if (cacheResult.isMatch) {
            return {
                isMatch: true,
                resolverName: cacheResult.resolverName,
            };
        }
        const repoPath = await this.gitCheckout.checkoutRepo({
            tag: pkg.version,
            commitSha: repo.commitSha,
            baseDir: workDir,
            url: repo.url,
        });
        const ciResult = await this.ciResolver.resolve({
            targetNode,
            repoPath,
            packageReleaseDate: pkg.releaseDate,
        });
        if (ciResult.isMatch) {
            return {
                isMatch: true,
                resolverName: cacheResult.resolverName,
            };
        }
        const testResult = await this.testResolver.resolve({
            repoPath,
        });
        if (testResult.isMatch) {
            return {
                isMatch: true,
                resolverName: cacheResult.resolverName,
            };
        }
        return {
            isMatch: false,
        };
    }
};
Flow = __decorate([
    inversify_1.injectable(),
    __metadata("design:paramtypes", [git_1.IGitCheckout,
        cacheResolver_1.ICacheResolver,
        ciResolver_1.ICIResolver,
        testResolver_1.ITestResolver])
], Flow);
exports.Flow = Flow;
