"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const specificCIResolver_1 = require("../../interfaces/specificCIResolver");
const inversify_1 = require("inversify");
const nodeModulesContainer_1 = require("../../../../container/nodeModulesContainer");
const yaml_1 = require("yaml");
const nodeVersionRegex = /node ([^\s]+)/i;
const nodeVersionInstallRegex = /Install-Product node ([^\s]+)/i;
const nvmInstallRegex = /nvm install ([^\s]+)/i;
const nodeEnvRegex = /\$env:(.+)/i;
const nodeVersionInstallMapper = (command) => {
    if (!command) {
        return;
    }
    const match = nodeVersionInstallRegex.exec(command) || nvmInstallRegex.exec(command);
    return match === null || match === void 0 ? void 0 : match[1];
};
const nodeVersionMapper = (command) => {
    const match = nodeVersionRegex.exec(command);
    return match === null || match === void 0 ? void 0 : match[1];
};
const emptyFilter = (command) => {
    return Boolean(command);
};
const psObjectMapper = (command) => {
    return command.ps || command.cmd || command.sh;
};
const envObjectMapper = (variable) => (command) => {
    return command[variable];
};
const parseStack = (stack, versions) => {
    if (!stack) {
        return;
    }
    stack
        .split(`,`)
        .map(nodeVersionMapper)
        .forEach((version) => {
        if (version) {
            versions.push(version);
        }
    });
};
const ciFileName = `.appveyor.yml`;
const resolverName = `appVeyor`;
let AppVeyorResolver = class AppVeyorResolver extends specificCIResolver_1.ISpecificCIResolver {
    constructor(fs) {
        super();
        this.fs = fs;
        this.resolverName = resolverName;
    }
    async resolve({ repoPath }) {
        var _a;
        const fileName = path.join(repoPath, ciFileName);
        let fileContents;
        try {
            fileContents = await this.fs.promises.readFile(fileName, `utf-8`);
        }
        catch (e) {
            return;
        }
        const versions = [];
        const yaml = yaml_1.parse(fileContents);
        const stack = yaml.stack;
        parseStack(stack, versions);
        const installCommands = yaml.install;
        if (!installCommands) {
            return versions;
        }
        const nodeVersion = installCommands
            .map(psObjectMapper)
            .map(nodeVersionInstallMapper)
            .filter(emptyFilter)[0];
        if (!nodeVersion) {
            return versions;
        }
        const match = nodeEnvRegex.exec(nodeVersion);
        if (!match) {
            versions.push(nodeVersion);
            return versions;
        }
        else {
            const matrixVarName = match[1];
            const environment = ((_a = yaml.environment) === null || _a === void 0 ? void 0 : _a.matrix) || yaml.environment;
            if (!environment) {
                return versions;
            }
            return versions.concat(environment.map(envObjectMapper(matrixVarName)).filter(emptyFilter));
        }
    }
};
AppVeyorResolver = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(nodeModulesContainer_1.TYPES.FS)),
    __metadata("design:paramtypes", [Object])
], AppVeyorResolver);
exports.AppVeyorResolver = AppVeyorResolver;
