"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const specificCIResolver_1 = require("../../interfaces/specificCIResolver");
const inversify_1 = require("inversify");
const nodeModulesContainer_1 = require("../../../../container/nodeModulesContainer");
const nodeVersionRegex = new RegExp(`node-version:\\s*(.+)`, `ig`);
const ciFilePath = path.join(`.github`, `workflows`);
const resolverName = `githubActions`;
let GithubActionsResolver = class GithubActionsResolver extends specificCIResolver_1.ISpecificCIResolver {
    constructor(fs) {
        super();
        this.fs = fs;
        this.resolverName = resolverName;
    }
    async resolve({ repoPath }) {
        const folderName = path.join(repoPath, ciFilePath);
        const foundVersions = new Set();
        let files;
        try {
            files = await this.fs.promises.readdir(folderName);
        }
        catch (e) {
            return;
        }
        for (const fileName of files) {
            const fileContents = await this.fs.promises.readFile(path.join(folderName, fileName), `utf-8`);
            let match;
            do {
                match = nodeVersionRegex.exec(fileContents);
                if (match) {
                    const versionsStr = match[1];
                    if (!versionsStr.startsWith(`$`)) {
                        let versions;
                        if (versionsStr.startsWith(`[`) && versionsStr.endsWith(`]`)) {
                            versions = versionsStr.substr(1, versionsStr.length - 2).split(`,`);
                        }
                        else {
                            versions = [versionsStr];
                        }
                        versions.forEach((version) => {
                            version = version.replace(/['"]/g, ``).trim();
                            foundVersions.add(version);
                        });
                    }
                }
            } while (match);
        }
        return Array.from(foundVersions);
    }
};
GithubActionsResolver = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(nodeModulesContainer_1.TYPES.FS)),
    __metadata("design:paramtypes", [Object])
], GithubActionsResolver);
exports.GithubActionsResolver = GithubActionsResolver;
