"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const targetMatcher_1 = require("../interfaces/targetMatcher");
const inversify_1 = require("inversify");
const lts_1 = require("../../../utils/lts");
const __1 = require("..");
// eslint-disable-next-line @typescript-eslint/quotes
const semver = require("semver");
const semverOptions = {
    loose: true,
};
const coerce = (version) => {
    const coercedAll = semver.coerce(version, semverOptions);
    if (!coercedAll) {
        return undefined;
    }
    const majorStr = coercedAll.major.toFixed(0);
    const coerced = semver.coerce(majorStr, semverOptions);
    return coerced.format();
};
let TargetMatcher = class TargetMatcher extends targetMatcher_1.ITargetMatcher {
    constructor(lts) {
        super();
        this.lts = lts;
    }
    async match({ targetNode, candidates, packageReleaseDate }) {
        const validTarget = coerce(targetNode);
        if (!validTarget) {
            throw new TypeError(`Node target version ${targetNode} is not valid`);
        }
        const resolvedCandidates = [];
        for (const candidate of candidates) {
            if (candidate === __1.LTS_VERSION) {
                const resolved = await this.lts.resolveLtsVersion({
                    date: packageReleaseDate,
                });
                Array.prototype.push.apply(resolvedCandidates, resolved);
            }
            else {
                resolvedCandidates.push(candidate);
            }
        }
        const matchingCandidates = resolvedCandidates.filter((candidate) => {
            const validCandidate = coerce(candidate);
            return validCandidate && semver.eq(validTarget, validCandidate);
        });
        return matchingCandidates.length > 0;
    }
};
TargetMatcher = __decorate([
    inversify_1.injectable(),
    __metadata("design:paramtypes", [lts_1.ILts])
], TargetMatcher);
exports.TargetMatcher = TargetMatcher;
