"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const inversify_1 = require("inversify");
const enginesResolver_1 = require("../interfaces/enginesResolver");
// eslint-disable-next-line @typescript-eslint/quotes
const semver = require("semver");
const semverOptions = {
    loose: true,
};
const coerceSemVer = (version) => {
    const majorStr = version.major.toFixed(0);
    return semver.coerce(majorStr, semverOptions);
};
const coerce = (version) => {
    const coercedAll = semver.coerce(version, semverOptions);
    if (!coercedAll) {
        return undefined;
    }
    const coerced = coerceSemVer(coercedAll);
    return coerced.format();
};
const strictSatisfiesRange = (target, range) => {
    const rangeObj = new semver.Range(range, semverOptions);
    const revisedComparatorRange = [];
    for (const comparatorRange of rangeObj.set) {
        let hasUpperBound = false;
        let hasLowerBound = false;
        for (const comparator of comparatorRange) {
            if (comparator.operator === `<` || comparator.operator === `<=`) {
                hasUpperBound = true;
            }
            else if (comparator.operator === `>` || comparator.operator === `>=`) {
                hasLowerBound = true;
            }
        }
        if ((hasLowerBound && hasUpperBound) || (!hasLowerBound && !hasUpperBound)) {
            revisedComparatorRange.push(comparatorRange);
        }
    }
    rangeObj.set = revisedComparatorRange;
    const revisedRange = new semver.Range(rangeObj.format(), semverOptions);
    return semver.satisfies(target, revisedRange);
};
const resolverName = `engines field`;
let EnginesResolver = class EnginesResolver extends enginesResolver_1.IEnginesResolver {
    constructor() {
        super();
    }
    async resolve({ engines, targetNode }) {
        if (!engines) {
            return {
                isMatch: false,
            };
        }
        const validTarget = coerce(targetNode);
        if (!validTarget) {
            throw new TypeError(`Node target version ${targetNode} is not valid`);
        }
        const validRange = semver.validRange(engines, semverOptions);
        if (!validRange) {
            throw new TypeError(`Engines range ${engines} is not valid`);
        }
        const isMatch = strictSatisfiesRange(validTarget, validRange);
        if (isMatch) {
            return {
                isMatch: true,
                resolverName,
            };
        }
        return {
            isMatch: false,
        };
    }
};
EnginesResolver = __decorate([
    inversify_1.injectable(),
    __metadata("design:paramtypes", [])
], EnginesResolver);
exports.EnginesResolver = EnginesResolver;
