"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const inversify_1 = require("inversify");
const nodeModulesContainer_1 = require("../../../container/nodeModulesContainer");
const logger_1 = require("../../logger");
let Git = class Git {
    constructor(git, loggerFactory) {
        this.git = git;
        this.logger = loggerFactory.getLogger(`Git`);
    }
    async checkoutCommit({ repo, commitSha }) {
        this.logger.info(`Checking out commit ${commitSha}`);
        try {
            await repo.checkout(commitSha);
        }
        catch (e) {
            this.logger.debug(`Failed Checking out commit ${commitSha}`, e);
            throw new Error(`Failed to checkout commit ${commitSha}`);
        }
        this.logger.success(`Checked out commit ${commitSha}`);
    }
    async cloneRepo({ url, dir }) {
        this.logger.info(`Cloning repo ${url}`);
        this.logger.debug(`Cloning repo to ${dir}`);
        const repo = this.git(dir);
        await repo.clone(url, dir);
        this.logger.success(`Cloned repo ${url}`);
        return repo;
    }
    async locateTag({ repo, tag }) {
        // @ts-ignore
        // eslint-disable-next-line no-underscore-dangle
        const repoName = repo._baseDir;
        this.logger.debug(`Locating all references of repo ${repoName}`);
        const refsResult = await repo.tags();
        const tagRefs = refsResult.all;
        this.logger.debug(`Located ${tagRefs.length} tags. Looking for tag ${tag}`);
        const filteredTags = tagRefs.filter((currentTag) => {
            return currentTag.includes(tag);
        });
        this.logger.debug(`Located ${filteredTags.length} matching tag(s)`);
        if (filteredTags.length > 1) {
            this.logger.debug(`Too many matching tags`);
            throw new Error(`Too many matching tags for tag ${tag} - [${filteredTags.join(`, `)}]`);
        }
        const tagRef = filteredTags[0];
        if (!tagRef) {
            this.logger.debug(`No matching tags`);
            throw new Error(`Failed to locate tag ${tag}`);
        }
        this.logger.debug(`Located matching tag ${tagRef.toString()}`);
        return tagRef;
    }
    async openRepo({ path }) {
        this.logger.info(`Opening existing repo in ${path}`);
        const repo = this.git(path);
        this.logger.success(`Opened repo`);
        return repo;
    }
};
Git = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(nodeModulesContainer_1.TYPES.SimpleGit)),
    __metadata("design:paramtypes", [Function, logger_1.ILoggerFactory])
], Git);
exports.Git = Git;
