"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const inversify_1 = require("inversify");
const gitCheckout_1 = require("../interfaces/gitCheckout");
const getRepoDirName_1 = require("./getRepoDirName");
const git_1 = require("./git");
const nodeModulesContainer_1 = require("../../../container/nodeModulesContainer");
const logger_1 = require("../../logger");
let GitCheckout = class GitCheckout extends gitCheckout_1.IGitCheckout {
    constructor(git, fs, loggerFactory) {
        super();
        this.git = git;
        this.fs = fs;
        this.logger = loggerFactory.getLogger(`GitCheckout`);
    }
    async checkoutRepo({ url, baseDir, tag, commitSha }) {
        const dirName = await getRepoDirName_1.getRepoDirName({ url });
        const fullDir = path.join(baseDir, dirName);
        this.logger.info(`Cloning repo ${url} to ${fullDir}`);
        let exists;
        try {
            await this.fs.promises.stat(fullDir);
            this.logger.debug(`Repo already exists locally`);
            exists = true;
        }
        catch (e) {
            this.logger.debug(`Repo doesn't exist locally, will create directory ${fullDir}`);
            await this.fs.promises.mkdir(fullDir);
            exists = false;
        }
        const repo = exists
            ? await this.git.openRepo({
                path: fullDir,
            })
            : await this.git.cloneRepo({
                url,
                dir: fullDir,
            });
        if (commitSha) {
            this.logger.debug(`Checking out provided commit ${commitSha}`);
            await this.git.checkoutCommit({
                repo,
                commitSha,
            });
        }
        else {
            this.logger.debug(`Looking for commit of tag ${tag}`);
            const reference = await this.git.locateTag({ repo, tag });
            this.logger.debug(`Checking out found commit ${reference} for tag ${tag}`);
            await this.git.checkoutCommit({
                repo,
                commitSha: reference,
            });
        }
        this.logger.success(`Checking out repo ${dirName}`);
        return fullDir;
    }
};
GitCheckout = __decorate([
    inversify_1.injectable(),
    __param(1, inversify_1.inject(nodeModulesContainer_1.TYPES.FS)),
    __metadata("design:paramtypes", [git_1.Git, Object, logger_1.ILoggerFactory])
], GitCheckout);
exports.GitCheckout = GitCheckout;
