"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cacheProp = Symbol.for(`[memoize]`);
const defaultKeyBuilder = (v) => {
    return v;
};
const getCache = (target) => {
    if (!target[cacheProp]) {
        Object.defineProperty(target, cacheProp, {
            value: Object.create(null),
            configurable: true,
        });
    }
    return target[cacheProp];
};
const getKeyCache = (target, key) => {
    const dict = getCache(target);
    if (!dict[key]) {
        dict[key] = new Map();
    }
    return dict[key];
};
const memoizeFn = (namespace, func, keyBuilder) => {
    return function (...args) {
        const cache = getKeyCache(this, namespace);
        const key = keyBuilder.apply(this, args);
        if (cache.has(key)) {
            return cache.get(key);
        }
        const res = func.apply(this, args);
        cache.set(key, res);
        return res;
    };
};
exports.memoize = (keyBuilder) => {
    return (_, propertyKey, descriptor) => {
        descriptor.value = memoizeFn(propertyKey, descriptor.value, keyBuilder || defaultKeyBuilder);
    };
};
