"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const inversify_1 = require("inversify");
const runner_1 = require("../interfaces/runner");
const nodeModulesContainer_1 = require("../../../container/nodeModulesContainer");
const logger_1 = require("../../logger");
let Runner = class Runner extends runner_1.IRunner {
    constructor(childProcess, loggerFactory) {
        super();
        this.childProcess = childProcess;
        this.logger = loggerFactory.getLogger(`Command Runner`);
    }
    async executeCommand({ command, execOptions }) {
        const firstCommand = command[0];
        const options = command.slice(1);
        if (this.logger.isDebugEnabled()) {
            this.logger.debug(`Running command ${command.join(` `)}`);
        }
        await this.promisifiedSpawn({ command: firstCommand, options, execOptions });
    }
    async promisifiedSpawn({ command, options, execOptions }) {
        return await new Promise((resolve, reject) => {
            const outputBuffer = [];
            const errorBuffer = [];
            const subProcess = this.childProcess.spawn(command, options, execOptions);
            subProcess.stdout.on(`data`, (data) => {
                if (this.logger.isDebugEnabled()) {
                    outputBuffer.push(data);
                }
            });
            subProcess.stderr.on(`data`, (data) => {
                errorBuffer.push(data);
            });
            subProcess.on(`error`, (err) => {
                this.logger.error(`spawn error`, err);
            });
            subProcess.on(`close`, (code) => {
                if (this.logger.isDebugEnabled()) {
                    const output = Buffer.concat(outputBuffer)
                        .toString()
                        .trim();
                    if (output) {
                        this.logger.debug(`Command output:\n`, output);
                    }
                    const errorOutput = Buffer.concat(errorBuffer)
                        .toString()
                        .trim();
                    if (errorOutput) {
                        this.logger.debug(`Command error output:\n`, errorOutput);
                    }
                }
                else if (code !== 0) {
                    const errorOutput = Buffer.concat(errorBuffer)
                        .toString()
                        .trim();
                    if (errorOutput) {
                        this.logger.error(`Command error output:\n`, errorOutput);
                    }
                }
                if (this.logger.isDebugEnabled()) {
                    this.logger.debug(`Command "${command} ${options.join(` `)}" finished with exit status ${code}`);
                }
                if (code === 0) {
                    resolve();
                }
                else {
                    reject(new Error(`"${command} ${options.join(` `)}" exited with code: ${code}`));
                }
            });
        });
    }
};
Runner = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(nodeModulesContainer_1.TYPES.ChildProcess)),
    __metadata("design:paramtypes", [Object, logger_1.ILoggerFactory])
], Runner);
exports.Runner = Runner;
